#ifndef TDDColorKeyH
#define TDDColorKeyH

// ==========================================================================
// File:         TDDColorKey.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDColorKey Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDColorKey
// Description: The TDDColorKey Component wraps the DirectX DDCOLORKEY 
//              structure, which defines color key or color range 
//              information. 
//              
//              It is used by TDx_DrawSurface::GetColorKey() and 
//              TDx_DrawSurface::SetColorKey() when retrieving or defining 
//              color key values. 
//              It is also used in the TDDBltFX, TDDOverlayFX and 
//              TDDSurfaceDesc components. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDColorKey : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDColorKey : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ColorSpaceHighValue
    // Description: The ColorSpaceHighValue property defines the upper value, 
    //              inclusive, of the color range used as the color key. 
    // ----------------------------------------------------------------------
    __property dword ColorSpaceHighValue = { read=FGetColorSpaceHighValue, 
    write=FSetColorSpaceHighValue, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ColorSpaceLowValue
    // Description: The ColorSpaceLowValue property defines the lower value, 
    //              inclusive, of the color range used as the color key. 
    // ----------------------------------------------------------------------
    __property dword ColorSpaceLowValue = { read=FGetColorSpaceLowValue, 
    write=FSetColorSpaceLowValue, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDColorKey component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDColorKey method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DDCOLORKEY structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDColorKey's internal DDCOLORKEY structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDColorKey(TComponent* Owner); 
    virtual __fastcall ~TDDColorKey(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDCOLORKEY* Internal_DDCOLORKEY_Ptr = { 
    read=FGetInternal_DDCOLORKEY_Ptr, nodefault }; 

    void __fastcall Internal_DDCOLORKEY_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetColorSpaceHighValue(); 
    void __fastcall FSetColorSpaceHighValue( dword pColorSpaceHighValue ); 
    dword __fastcall FGetColorSpaceLowValue(); 
    void __fastcall FSetColorSpaceLowValue( dword pColorSpaceLowValue ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDCOLORKEY* __fastcall FGetInternal_DDCOLORKEY_Ptr(); 
    void __fastcall FSetInternal_DDCOLORKEY( DDCOLORKEY* pDDCOLORKEY ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDCOLORKEY fDDCOLORKEY; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
