#ifndef TDDSCapsH
#define TDDSCapsH

// ==========================================================================
// File:         TDDSCaps.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDSCaps Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"

// ==========================================================================
// Set:         TDDSCapsCaps_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDSCaps::Caps property using the BCB Object Inspector. For 
//              detailed descriptions of each flag, see the Caps property 
//              description below. 
// ==========================================================================
enum TDDSCaps_Caps_Flags
    {
    ddscaps_3ddevice,
    ddscaps_alpha,
    ddscaps_backbuffer,
    ddscaps_complex,
    ddscaps_flip,
    ddscaps_frontbuffer,
    ddscaps_hwcodec,
    ddscaps_livevideo,
    ddscaps_localvidmem,
    ddscaps_mipmap,
    ddscaps_modex,
    ddscaps_nonlocalvidmem,
    ddscaps_offscreenplain,
    ddscaps_overlay,
    ddscaps_owndc,
    ddscaps_palette,
    ddscaps_primarysurface,
    ddscaps_standardvgamode,
    ddscaps_systemmemory,
    ddscaps_texture,
    ddscaps_videomemory,
    ddscaps_videoport,
    ddscaps_visible,
    ddscaps_writeonly,
    ddscaps_zbuffer
    };
typedef Set< TDDSCaps_Caps_Flags, 
             ddscaps_3ddevice, 
             ddscaps_zbuffer >  TDDSCaps_Caps_Set;
// ==========================================================================
// Set:         TDDSCapsCaps2_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDSCaps::Caps2 property using the BCB Object Inspector. For 
//              detailed descriptions of each flag, see the Caps2 property 
//              description below. 
// ==========================================================================
enum TDDSCaps_Caps2_Flags
    {
    ddscaps2_cubemap,
    ddscaps2_cubemap_allfaces,
    ddscaps2_cubemap_negativex,
    ddscaps2_cubemap_negativey,
    ddscaps2_cubemap_negativez,
    ddscaps2_cubemap_positivex,
    ddscaps2_cubemap_positivey,
    ddscaps2_cubemap_positivez,
    ddscaps2_d3dtexturemanage,
    ddscaps2_donotpersist,
    ddscaps2_hardwaredeinterlace,
    ddscaps2_hintantialiasing,
    ddscaps2_hintdynamic,
    ddscaps2_hintstatic,
    ddscaps2_mipmapsublevel,
    ddscaps2_opaque,
    ddscaps2_stereosurfaceleft,
    ddscaps2_texturemanage
    };
typedef Set< TDDSCaps_Caps2_Flags, 
             ddscaps2_cubemap, 
             ddscaps2_texturemanage >  TDDSCaps_Caps2_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDSCaps
// Description: The TDDSCaps component wraps the DirectX DDSCAPS2 structure, 
//              which defines the capabilities of a surface. 
//              
//              It is used by TDx_Draw::GetAvailableVidMem(), 
//              TDx_DrawSurface::GetAttachedSurface() and 
//              TDx_DrawSurface::GetCaps() to define specific surface 
//              capabilties. 
//              It is also part of the TDDCaps and TDDSurfaceDesc components. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDSCaps : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDSCaps : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps_
    // Description: The Caps property defines flags indicating the 
    //              capabilities of a surface. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Caps' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Caps 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDSCaps_Caps_Set Caps_ = { read=FGetCaps_, write=FSetCaps_, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Caps2_
    // Description: The Caps2 property defines flags indicating more surface 
    //              capabilities as well as some usage hints for the driver. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Caps2' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Caps2 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDSCaps_Caps2_Set Caps2_ = { read=FGetCaps2_, 
    write=FSetCaps2_, nodefault }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDSCaps component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Caps
    // Description: The Caps property defines flags indicating the 
    //              capabilities of a surface. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDSCAPS_3DDEVICE -
    //                The surface this component describes can be used for 3D 
    //                rendering. 
    //                
    //                This flag can be used to ensure devices capable of only 
    //                rendering to a certain heap will have off screen 
    //                surfaces allocated from the appropriate heap. 
    //                
    //                Setting this flag for a heap will make sure the surface 
    //                is not allocated from it. 
    //              DDSCAPS_ALPHA -
    //                The surface this component describes is an alpha only 
    //                surface. 
    //              DDSCAPS_BACKBUFFER -
    //                The surface this component describes is the back buffer 
    //                of a flipping construct. 
    //                
    //                TDx_Draw::CreateSurface() usually sets this flag when 
    //                called with DDSCAPS_FLIP set. 
    //                This flag will only be set on the surface immediately 
    //                following a DDSCAPS_FRONTBUFFER surface in a flip 
    //                chain. 
    //                Other back buffers can be identified by having a 
    //                DDSCAPS_FLIP flag, their attachment order and the 
    //                absence of the DDSCAPS_FRONTBUFFER or 
    //                DDSCAPS_BACKBUFFER flags. 
    //                
    //                Using TDx_Draw::CreateSurface() with this flag set 
    //                creates a stand alone back buffer which may then be 
    //                attached with 
    //                TDx_DrawSurface::AddAttachedSurface() to front, back or 
    //                both buffers to form a flipping construct. 
    //                An arbitrary number of flipping chain surfaces are 
    //                supported. 
    //              DDSCAPS_COMPLEX -
    //                The surface this component describes is a complex 
    //                surface. 
    //              DDSCAPS_FLIP -
    //                The surface this component describes is a part of a 
    //                flipping construct. 
    //                
    //                Pass this flag to TDx_Draw::CreateSurface() to create a 
    //                front buffer and one or more back buffers. 
    //                The DDSCAPS_FRONTBUFFER and DDSCAPS_BACKBUFFER flags 
    //                will be set on the frontbuffer surface and the next 
    //                surface in the chain respectively. 
    //                The TDDSurfaceDesc::BackBufferCount used in the 
    //                creation call must be at least 1 in order for the call 
    //                to succeed. 
    //                
    //                DDSCAPS_COMPLEX must always be set when creating 
    //                multiple surfaces using TDx_Draw::CreateSurface(). 
    //              DDSCAPS_FRONTBUFFER -
    //                The surface this component describes is the front 
    //                buffer of a flipping construct. 
    //                
    //                This flag is usually set by TDx_Draw::CreateSurface() 
    //                when called using the DDSCAPS_FLIP flag. 
    //                
    //                When TDx_Draw::CreateSurface() is called with this flag 
    //                set, a stand-alone front buffer is created. It will not 
    //                have DDSCAPS_FLIP set and it can be attached to other 
    //                back buffers to form a flipping construct by using 
    //                TDx_DrawSurface::AddAttachedSurface(). 
    //              DDSCAPS_HWCODEC -
    //                The surface this component describes can have a data 
    //                stream decompressed to it by the hardware. 
    //                
    //                This would generally be a stream from a video port to 
    //                an overlay surface but other uses are possible. 
    //              DDSCAPS_LIVEVIDEO -
    //                The surface this component describes is capable of 
    //                receiving live video. 
    //              DDSCAPS_LOCALVIDMEM -
    //                The surface this component describes exists in local 
    //                video memory. 
    //                DDSCAPS_VIDEOMEMORY must also be set. 
    //                This flag cannot be used with DDSCAPS_NONLOCALVIDMEM. 
    //              DDSCAPS_MIPMAP -
    //                The surface this component describes is a mipmap level. 
    //                
    //                Other DDSCAPS_MIPMAP surfaces can be attached to form 
    //                the mipmap by either creating a number of surfaces and 
    //                attaching them using 
    //                TDx_DrawSurface::AddAttachedSurface() or all at once 
    //                using TDx_Draw::CreateSurface() to create a complex 
    //                mipmap surface chain. 
    //                
    //                DDSCAPS_TEXTURE must also be set. 
    //              DDSCAPS_MODEX -
    //                The surface this component describes is a 320 by 200 or 
    //                320 by 240 Mode X surface. 
    //              DDSCAPS_NONLOCALVIDMEM -
    //                The surface this component describes exists in nonlocal 
    //                video memory. 
    //                DDSCAPS_VIDEOMEMORY must also be set. 
    //                This flag cannot be used with DDSCAPS_LOCALVIDMEM. 
    //              DDSCAPS_OFFSCREENPLAIN -
    //                The surface this component describes is an ordinary off 
    //                screen surface. 
    //                Not an overlay, texture, z-buffer, front-buffer, 
    //                back-buffer or alpha surface. 
    //              DDSCAPS_OVERLAY -
    //                The surface this component describes is an overlay. 
    //                DDSCAPS_VISIBLE can be used to determine if the overlay 
    //                is visible. 
    //              DDSCAPS_OWNDC -
    //                The surface this component describes has a device 
    //                context assigned to it. 
    //              DDSCAPS_PALETTE -
    //                The device driver allows unique palettes to be created 
    //                and attached to the surface this component describes. 
    //              DDSCAPS_PRIMARYSURFACE -
    //                The surface this component describes is the primary 
    //                surface. 
    //              DDSCAPS_STANDARDVGAMODE -
    //                The surface this component describes is a standard VGA 
    //                mode surface. 
    //                This flag cannot be used with DDSCAPS_MODEX. 
    //              DDSCAPS_SYSTEMMEMORY -
    //                The surface this component describes exists in system 
    //                memory. 
    //              DDSCAPS_TEXTURE -
    //                The surface this component describes can be used as a 
    //                3D texture. 
    //                This flag being set does not neccessarily mean the 
    //                surface is being used as a 3D texture. 
    //              DDSCAPS_VIDEOMEMORY -
    //                The surface this component describes exists in display 
    //                memory. 
    //              DDSCAPS_VIDEOPORT -
    //                The surface this component describes can receive data 
    //                from a video port. 
    //              DDSCAPS_VISIBLE -
    //                Changes made to the surface this component describes 
    //                will immediately be visible. 
    //                This flag is always set for the primary surface, for 
    //                overlays when they are being overlaid and texture maps 
    //                while they are being textured. 
    //              DDSCAPS_WRITEONLY -
    //                Write access only is permitted for the surface this 
    //                component describes. 
    //                Trying to read this surface may generate general 
    //                protection faults and the read results will not be 
    //                meaningful. 
    //              DDSCAPS_ZBUFFER -
    //                The surface this component describes is a z-buffer. 
    // ----------------------------------------------------------------------
    __property dword Caps = { read=FGetCaps, write=FSetCaps, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Caps2
    // Description: The Caps2 property defines flags indicating more surface 
    //              capabilities as well as some usage hints for the driver. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDSCAPS2_CUBEMAP -
    //                The surface is a cubic environment map. 
    //                The various DDSCAPS2_CUBEMAP_???? flags can be used to 
    //                indicate which face or faces of the cubemap to create 
    //                or access. 
    //              DDSCAPS2_CUBEMAP_ALLFACES -
    //                Create or access all the faces of a cubic environment 
    //                map. 
    //                DDSCAPS2_CUBEMAP must also be set. 
    //              DDSCAPS2_CUBEMAP_NEGATIVEX -
    //                Create or access the negative X face of a cubic 
    //                environment map. 
    //                DDSCAPS2_CUBEMAP must also be set. 
    //              DDSCAPS2_CUBEMAP_NEGATIVEY -
    //                Create or access the negative Y face of a cubic 
    //                environment map. 
    //                DDSCAPS2_CUBEMAP must also be set. 
    //              DDSCAPS2_CUBEMAP_NEGATIVEZ -
    //                Create or access the negative Z face of a cubic 
    //                environment map. 
    //                DDSCAPS2_CUBEMAP must also be set. 
    //              DDSCAPS2_CUBEMAP_POSITIVEX -
    //                Create or access the positive X face of a cubic 
    //                environment map. 
    //                DDSCAPS2_CUBEMAP must also be set. 
    //              DDSCAPS2_CUBEMAP_POSITIVEY -
    //                Create or access the positive Y face of a cubic 
    //                environment map. 
    //                DDSCAPS2_CUBEMAP must also be set. 
    //              DDSCAPS2_CUBEMAP_POSITIVEZ -
    //                Create or access the positive Z face of a cubic 
    //                environment map. 
    //                DDSCAPS2_CUBEMAP must also be set. 
    //              DDSCAPS2_D3DTEXTUREMANAGE -
    //                The surface this component describes will be 
    //                automatically managed by the driver or the TDx_3D 
    //                texture manager. 
    //                This setting will only be valid if DDSCAPS_TEXTURE is 
    //                set in TDDSCaps::Caps. 
    //                This flag should not be set when using TDx_3DR. 
    //              DDSCAPS2_DONOTPERSIST -
    //                Indicates that the managed surface can be lost without 
    //                causing problems. 
    //              DDSCAPS2_HARDWAREDEINTERLACE -
    //                The surface this component describes will receive 
    //                de-interlaced video port data. 
    //                To utilize this capability DDSCAPS_VIDEOPORT and 
    //                DDSCAPS_OVERLAY must be set in TDDSCaps::Caps. 
    //              DDSCAPS2_HINTANTIALIASING -
    //                The surface this component describes will use 
    //                antialiasing. 
    //                This setting will only be valid if DDSCAPS_3DDEVICE is 
    //                set in TDDSCaps::Caps. 
    //              DDSCAPS2_HINTDYNAMIC -
    //                The surface this component describes will be be locked 
    //                frequently. 
    //                This setting will only be valid if DDSCAPS_TEXTURE is 
    //                set in TDDSCaps::Caps. 
    //                The DDSCAPS2_HINTSTATIC and DDSCAPS2_OPAQUE flags must 
    //                not be set. 
    //              DDSCAPS2_HINTSTATIC -
    //                The surface this component describes can be retiled or 
    //                reordered when loading. 
    //                This setting will only be valid if DDSCAPS_TEXTURE is 
    //                set in TDDSCaps::Caps. 
    //                The DDSCAPS2_HINTDYNAMIC and DDSCAPS2_OPAQUE flags must 
    //                not be set. 
    //              DDSCAPS2_MIPMAPSUBLEVEL -
    //                All sub level surfaces of a complex construct such as a 
    //                mipmapped cube map should have this flag set. 
    //                This allows TDx_DrawSurface::GetAttachedSurface() to 
    //                distinguish between the top level faces and the 
    //                attached mipmap levels, allowing retrieval via that 
    //                method rather than relying on 
    //                TDx_DrawSurface::EnumAttachedSurfaces(). 
    //              DDSCAPS2_OPAQUE -
    //                The surface this component describes will never be 
    //                locked again. 
    //                This setting will only be valid if DDSCAPS_TEXTURE flag 
    //                is set in TDDSCaps::Caps. 
    //                The DDSCAPS2_HINTDYNAMIC and DDSCAPS2_HINTSTATIC flags 
    //                must not be set. 
    //                The driver may optimize this surface as desired. 
    //                Any attempts to lock or blit a surface with this flag 
    //                set will fail. 
    //              DDSCAPS2_STEREOSURFACELEFT -
    //                The surface is part of a stereo flipping chain. 
    //                Creating surfaces using this flag will result in a pair 
    //                of stereo surfaces to be created for each buffer in the 
    //                primary flipping chain. 
    //                Since TDx_DrawSurface::Flip() requires back buffers, 
    //                this means that at least four surfaces must be created. 
    //                If this flag is set in the TDDSurfaceDesc component 
    //                returned by TDx_Draw::EnumDisplayModes() or 
    //                TDx_Draw::GetDisplayMode(), it indicates that stereo is 
    //                supported for that mode. 
    //              DDSCAPS2_TEXTUREMANAGE -
    //                The surface this component describes is to be 
    //                automatically managed by the TDx_3D texture manager. 
    //                This setting will only be valid if DDSCAPS_TEXTURE is 
    //                set in DDSCaps::Caps. 
    // ----------------------------------------------------------------------
    __property dword Caps2 = { read=FGetCaps2, write=FSetCaps2, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDSCaps method or fget/fset. 
    //              eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DDSCAPS2 structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDSCaps's internal DDSCAPS2 structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Caps_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Caps property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Caps_Strings = { read=FGetCaps_Strings };

    // ----------------------------------------------------------------------
    // Property:    Caps2_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Caps2 property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Caps2_Strings = { read=FGetCaps2_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDSCaps(TComponent* Owner); 
    virtual __fastcall ~TDDSCaps(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDSCAPS2* Internal_DDSCAPS2_Ptr = { 
    read=FGetInternal_DDSCAPS2_Ptr, nodefault }; 

    void __fastcall Internal_DDSCAPS2_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDSCaps_Caps_Set __fastcall FGetCaps_(); 
    void __fastcall FSetCaps_( TDDSCaps_Caps_Set pCaps_ ); 
    TDDSCaps_Caps2_Set __fastcall FGetCaps2_(); 
    void __fastcall FSetCaps2_( TDDSCaps_Caps2_Set pCaps2_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetCaps(); 
    void __fastcall FSetCaps( dword pCaps ); 
    dword __fastcall FGetCaps2(); 
    void __fastcall FSetCaps2( dword pCaps2 ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetCaps_Strings();

    TStringList* __fastcall FGetCaps2_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDSCAPS2* __fastcall FGetInternal_DDSCAPS2_Ptr(); 
    void __fastcall FSetInternal_DDSCAPS2( DDSCAPS2* pDDSCAPS2 ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fCaps_Strings;
    TStringList* fCaps2_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDSCAPS2 fDDSCAPS2; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
