#ifndef TDDSurfaceDescH
#define TDDSurfaceDescH

// ==========================================================================
// File:         TDDSurfaceDesc.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDSurfaceDesc Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDDSurfaceDesc properties.
#include "TDDColorKey.H"
#include "TDDPixelFormat.H"
#include "TDDSCaps.H"
#include "TDx_DrawSurface.H"

// ==========================================================================
// Set:         TDDSurfaceDescFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDSurfaceDesc::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDDSurfaceDesc_Flags_Flags
    {
    ddsd_alphabitdepth,
    ddsd_backbuffercount,
    ddsd_caps,
    ddsd_ckdestblt,
    ddsd_ckdestoverlay,
    ddsd_cksrcblt,
    ddsd_cksrcoverlay,
    ddsd_height,
    ddsd_linearsize,
    ddsd_lpsurface,
    ddsd_mipmapcount,
    ddsd_pitch,
    ddsd_pixelformat,
    ddsd_refreshrate,
    ddsd_texturestage,
    ddsd_width
    };
typedef Set< TDDSurfaceDesc_Flags_Flags, 
             ddsd_alphabitdepth, 
             ddsd_width >  TDDSurfaceDesc_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDSurfaceDesc
// Description: The TDDSurfaceDesc component wraps the DirectX DDSURFACEDESC2 
//              structure, which describes the properties of a surface. 
//              
//              It is used in TDx_Draw::CreateSurface(), 
//              TDx_Draw::EnumDisplayModes(), TDx_Draw::EnumSurfaces() and 
//              TDx_Draw::GetDisplayMode(). Also in 
//              TDx_DrawSurface::GetSurfaceDesc(), 
//              TDx_DrawSurface::Initialize(), TDx_DrawSurface::Lock() and 
//              TDx_DrawSurface::SetSurfaceDesc(). The 
//              TDx_Draw::OnEnumModesCallback() and 
//              TDx_Draw::OnEnumSurfacesCallback events also utilize this 
//              component. 
//              
//              The relevant properties that need to be set may differ for 
//              each potential type of surface that can be created. 
//              
//              When calling TDx_DrawSurface::GetSurfaceDesc(), the Pitch and 
//              Surface properties are output values. 
//              When creating or modifying surfaces in memory allocated by 
//              the calling application, these properties should contain the 
//              pitch and location of that memory. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDSurfaceDesc : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDSurfaceDesc : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating which 
    //              properties of this component contain valid data. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDSurfaceDesc_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    AlphaBitDepth
    // Description: The AlphaBitDepth property defines the bit depth of the 
    //              alpha channel. 
    // ----------------------------------------------------------------------
    __property dword AlphaBitDepth = { read=FGetAlphaBitDepth, 
    write=FSetAlphaBitDepth, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    BackBufferCount
    // Description: The BackBufferCount property defines the number of back 
    //              buffers associated with the surface. 
    // ----------------------------------------------------------------------
    __property dword BackBufferCount = { read=FGetBackBufferCount, 
    write=FSetBackBufferCount, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    CKDestBlt
    // Description: The CKDestBlt property references the destination color 
    //              key for blit operations. 
    // ----------------------------------------------------------------------
    __property TDDColorKey* CKDestBlt = { read=FGetCKDestBlt, 
    write=FSetCKDestBlt, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    CKDestOverlay
    // Description: The CKDestOverlay property references the destination 
    //              color key for overlay operations. 
    // ----------------------------------------------------------------------
    __property TDDColorKey* CKDestOverlay = { read=FGetCKDestOverlay, 
    write=FSetCKDestOverlay, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    CKSrcBlt
    // Description: The CKSrcBlt property references the source color key for 
    //              blit operations. 
    // ----------------------------------------------------------------------
    __property TDDColorKey* CKSrcBlt = { read=FGetCKSrcBlt, 
    write=FSetCKSrcBlt, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    CKSrcOverlay
    // Description: The CKSrcOverlay property references the source color key 
    //              for overlay operations. 
    // ----------------------------------------------------------------------
    __property TDDColorKey* CKSrcOverlay = { read=FGetCKSrcOverlay, 
    write=FSetCKSrcOverlay, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    EmptyFaceColor
    // Description: Defines the color used to fill in cube map faces that 
    //              have not had a texture assigned to them. 
    // ----------------------------------------------------------------------
    __property dword EmptyFaceColor = { read=FGetEmptyFaceColor, 
    write=FSetEmptyFaceColor, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Height
    // Description: The Height property defines the vertical dimension, in 
    //              pixels, of the surface being described. 
    // ----------------------------------------------------------------------
    __property dword Height = { read=FGetHeight, write=FSetHeight, default=0 
    }; 

    // ----------------------------------------------------------------------
    // Property:    LinearSize
    // Description: The LinearSize property defines the buffer size. 
    //              Currently only used for compressed texture surfaces. 
    // ----------------------------------------------------------------------
    __property dword LinearSize = { read=FGetLinearSize, 
    write=FSetLinearSize, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MipMapCount
    // Description: The MipMapCount property defines the number of mipmap 
    //              levels. 
    // ----------------------------------------------------------------------
    __property dword MipMapCount = { read=FGetMipMapCount, 
    write=FSetMipMapCount, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Pitch
    // Description: The Pitch property defines the distance, in bytes, 
    //              between the start of one bitmap line and the start of the 
    //              next. 
    //              For TDx_DrawSurface::GetSurfaceDesc(), this is a return 
    //              value. 
    //              For TDx_DrawSurface::SetSurfaceDesc(), this is an input 
    //              value that must be a dword multiple. 
    // ----------------------------------------------------------------------
    __property long Pitch = { read=FGetPitch, write=FSetPitch, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PixelFormat
    // Description: The PixelFormat property references a TDDPixelFormat 
    //              component describing the surface's pixel format. 
    // ----------------------------------------------------------------------
    __property TDDPixelFormat* PixelFormat = { read=FGetPixelFormat, 
    write=FSetPixelFormat, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    RefreshRate
    // Description: The RefreshRate property defines the refresh rate of the 
    //              current display mode. 
    //              A value of 0 indicates an adapter fault. 
    // ----------------------------------------------------------------------
    __property dword RefreshRate = { read=FGetRefreshRate, 
    write=FSetRefreshRate, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    SCaps
    // Description: The SCaps property references a TDDSCaps component 
    //              defining the capabilities of the surface. 
    // ----------------------------------------------------------------------
    __property TDDSCaps* SCaps = { read=FGetSCaps, write=FSetSCaps, 
    default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDSURFACEDESC2) }; 

    // ----------------------------------------------------------------------
    // Property:    Surface
    // Description: The Surface property references the memory associated 
    //              with the surface this component describes. 
    //              For TDx_DrawSurface::Lock(), this should be a valid 
    //              pointer to surface memory. 
    //              For TDx_DrawSurface::SetSurfaceDesc(), this should be a 
    //              pointer to system memory explicitly allocated for the 
    //              surface by the calling application. 
    // ----------------------------------------------------------------------
    __property TDx_DrawSurface* Surface = { read=FGetSurface, 
    write=FSetSurface, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    TextureStage
    // Description: The TextureStage property defines the stage identifier 
    //              which binds the texture surface described by this 
    //              component to a specific stage of a 3D device's 
    //              multitexture cascade. 
    //              This setting is reccomended for most 3D hardware. 
    // ----------------------------------------------------------------------
    __property dword TextureStage = { read=FGetTextureStage, 
    write=FSetTextureStage, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Width
    // Description: The Width property defines the horizontal dimension, in 
    //              pixels, of the surface being described. 
    // ----------------------------------------------------------------------
    __property dword Width = { read=FGetWidth, write=FSetWidth, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDSurfaceDesc component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDSurfaceDesc method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating which 
    //              properties of this component contain valid data. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDSD_ALL -
    //                All properties of this component contain valid data. 
    //              DDSD_ALPHABITDEPTH -
    //                TDDSurfaceDesc::AlphaBitDepth contains valid data. 
    //              DDSD_BACKBUFFERCOUNT -
    //                TDDSurfaceDesc::BackBufferCount contains valid data. 
    //              DDSD_CAPS -
    //                TDDSurfaceDesc::SCaps contains valid data. 
    //              DDSD_CKDESTBLT -
    //                TDDSurfaceDesc::CKDestBlt contains valid data. 
    //              DDSD_CKDESTOVERLAY -
    //                TDDSurfaceDesc::CKDestOverlay contains valid data. 
    //              DDSD_CKSRCBLT -
    //                TDDSurfaceDesc::CKSrcBlt contains valid data. 
    //              DDSD_CKSRCOVERLAY -
    //                TDDSurfaceDesc::CKSrcOverlay contains valid data. 
    //              DDSD_HEIGHT -
    //                TDDSurfaceDesc::Height contains valid data. 
    //              DDSD_LINEARSIZE -
    //                TDDSurfaceDesc::LinearSize contains valid data. 
    //              DDSD_LPSURFACE -
    //                TDDSurfaceDesc::Surface contains valid data. 
    //              DDSD_MIPMAPCOUNT -
    //                TDDSurfaceDesc::MipMapCount contains valid data. 
    //              DDSD_PITCH -
    //                TDDSurfaceDesc::Pitch contains valid data. 
    //              DDSD_PIXELFORMAT -
    //                TDDSurfaceDesc::PixelFormat contains valid data. 
    //              DDSD_REFRESHRATE -
    //                TDDSurfaceDesc::RefreshRate contains valid data. 
    //              DDSD_TEXTURESTAGE -
    //                TDDSurfaceDesc::TextureStage contains valid data. 
    //              DDSD_WIDTH -
    //                TDDSurfaceDesc::Width contains valid data. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDSurfaceDesc's internal DDSURFACEDESC2 structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDSurfaceDesc(TComponent* Owner); 
    virtual __fastcall ~TDDSurfaceDesc(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDSURFACEDESC2* Internal_DDSURFACEDESC2_Ptr = { 
    read=FGetInternal_DDSURFACEDESC2_Ptr, nodefault }; 

    void __fastcall Internal_DDSURFACEDESC2_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDSurfaceDesc_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDDSurfaceDesc_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetAlphaBitDepth(); 
    void __fastcall FSetAlphaBitDepth( dword pAlphaBitDepth ); 
    dword __fastcall FGetBackBufferCount(); 
    void __fastcall FSetBackBufferCount( dword pBackBufferCount ); 
    TDDColorKey* __fastcall FGetCKDestBlt(); 
    void __fastcall FSetCKDestBlt( TDDColorKey* pCKDestBlt ); 
    TDDColorKey* __fastcall FGetCKDestOverlay(); 
    void __fastcall FSetCKDestOverlay( TDDColorKey* pCKDestOverlay ); 
    TDDColorKey* __fastcall FGetCKSrcBlt(); 
    void __fastcall FSetCKSrcBlt( TDDColorKey* pCKSrcBlt ); 
    TDDColorKey* __fastcall FGetCKSrcOverlay(); 
    void __fastcall FSetCKSrcOverlay( TDDColorKey* pCKSrcOverlay ); 
    dword __fastcall FGetEmptyFaceColor(); 
    void __fastcall FSetEmptyFaceColor( dword pEmptyFaceColor ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    dword __fastcall FGetHeight(); 
    void __fastcall FSetHeight( dword pHeight ); 
    dword __fastcall FGetLinearSize(); 
    void __fastcall FSetLinearSize( dword pLinearSize ); 
    dword __fastcall FGetMipMapCount(); 
    void __fastcall FSetMipMapCount( dword pMipMapCount ); 
    long __fastcall FGetPitch(); 
    void __fastcall FSetPitch( long pPitch ); 
    TDDPixelFormat* __fastcall FGetPixelFormat(); 
    void __fastcall FSetPixelFormat( TDDPixelFormat* pPixelFormat ); 
    dword __fastcall FGetRefreshRate(); 
    void __fastcall FSetRefreshRate( dword pRefreshRate ); 
    TDDSCaps* __fastcall FGetSCaps(); 
    void __fastcall FSetSCaps( TDDSCaps* pSCaps ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    TDx_DrawSurface* __fastcall FGetSurface(); 
    void __fastcall FSetSurface( TDx_DrawSurface* pSurface ); 
    dword __fastcall FGetTextureStage(); 
    void __fastcall FSetTextureStage( dword pTextureStage ); 
    dword __fastcall FGetWidth(); 
    void __fastcall FSetWidth( dword pWidth ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDSURFACEDESC2* __fastcall FGetInternal_DDSURFACEDESC2_Ptr(); 
    void __fastcall FSetInternal_DDSURFACEDESC2( DDSURFACEDESC2* 
    pDDSURFACEDESC2 ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    TDDColorKey* fCKDestBlt; 
    TDDColorKey* fCKDestOverlay; 
    TDDColorKey* fCKSrcBlt; 
    TDDColorKey* fCKSrcOverlay; 
    HRESULT fErrorValue; 
    TDDPixelFormat* fPixelFormat; 
    TDDSCaps* fSCaps; 
    TDx_DrawSurface* fSurface; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDSURFACEDESC2 fDDSURFACEDESC2; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
