#ifndef TDDVideoPortConnectH
#define TDDVideoPortConnectH

// ==========================================================================
// File:         TDDVideoPortConnect.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDVideoPortConnect Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"

// ==========================================================================
// Set:         TDDVideoPortConnectFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDDVideoPortConnect::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDDVideoPortConnect_Flags_Flags
    {
    ddvpconnect_discardsvrefdata,
    ddvpconnect_doubleclock,
    ddvpconnect_halfline,
    ddvpconnect_interlaced,
    ddvpconnect_invertpolarity,
    ddvpconnect_shareeven,
    ddvpconnect_shareodd,
    ddvpconnect_vact
    };
typedef Set< TDDVideoPortConnect_Flags_Flags, 
             ddvpconnect_discardsvrefdata, 
             ddvpconnect_vact >  TDDVideoPortConnect_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDVideoPortConnect
// Description: The TDDVideoPortConnect component wraps multiple 
//              DDVIDEOPORTCONNECT structures, wihch describe a video port 
//              connection. 
//              
//              This component is used by 
//              TDx_DrawVideoPortContainer::GetVideoPortConnectInfo(). 
//              It is also part of the TDDVideoPortStatus and 
//              TDDVideoPortDesc components. 
//              
//              To setup this component, first set the 
//              TDDVideoPortConnect::ArraySize property, then set each 
//              property using array indexes, eg: 
//              
//                  DDVideoPortConnect1->ArraySize = 1; 
//                  DDVideoPortConnect1->Flags[0] = DDVPCONNECT_INTERLACED; 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDVideoPortConnect : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDVideoPortConnect : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDVideoPortConnect component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              DDVIDEOPORTCONNECT structures wrapped inside the 
    //              TDDVideoPortConnect component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TDDVideoPortConnect component wraps a single 
    //              DDVIDEOPORTCONNECT structure. To wrap multiple 
    //              structures, set the TDDVideoPortConnect::ArraySize to the 
    //              total number of internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDVideoPortConnect method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating the video 
    //              port connection's capabilities. 
    //              The described effect applies when the flag is set. 
    // Flags:       DDVPCONNECT_DISCARDSVREFDATA -
    //                The video port will discard any data written during the 
    //                VREF period. 
    //              DDVPCONNECT_DOUBLECLOCK -
    //                Support for double clocking data is available or the 
    //                video port should perform double clocking. 
    //                This flag is only valid when using an external sync. 
    //              DDVPCONNECT_HALFLINE -
    //                The video port will write half lines into the frame 
    //                buffer. 
    //                This can sometimes cause the data to be displayed 
    //                incorrectly. 
    //              DDVPCONNECT_INTERLACED -
    //                The video signal is interlaced. 
    //                This flag is only used when creating a video port 
    //                component. 
    //              DDVPCONNECT_INVERTPOLARITY -
    //                Support for inverting field polarities is available or 
    //                the video port should perform polarity inversion. 
    //                This reverses the odd and even fields of a video 
    //                signal. 
    //              DDVPCONNECT_SHAREEVEN -
    //                Support for sharing the video port is available and 
    //                this video port component will use the even fields for 
    //                its data. 
    //                This flag is only used when creating a video port 
    //                component. 
    //              DDVPCONNECT_SHAREODD -
    //                Support for sharing the video port is available and 
    //                this video port component will use the odd fields for 
    //                its data. 
    //                This flag is only used when creating a video port 
    //                component. 
    //              DDVPCONNECT_VACT -
    //                Support for an external VACT signal is available or the 
    //                video port should use the external VACT signal. 
    //                This flag is only valid when using an external sync. 
    // ----------------------------------------------------------------------
    __property dword Flags[dword pArrayIndex] = { read=FGetFlags, 
    write=FSetFlags }; 

    // ----------------------------------------------------------------------
    // Property:    PortWidth
    // Description: The PortWidth property defines the physical pin width of 
    //              the video port. 
    //              This value does not represent the width of a surface in 
    //              memory. 
    //              This property must always be set, even when the port type 
    //              indicated by TDDVideoPortConnect::TypeID assumes a 
    //              certain size. 
    // ----------------------------------------------------------------------
    __property dword PortWidth[dword pArrayIndex] = { read=FGetPortWidth, 
    write=FSetPortWidth }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size[dword pArrayIndex] = { read=FGetSize, 
    write=FSetSize }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal DDVIDEOPORTCONNECT structures. This result is 
    //              the same as multiplying TDDVideoPortConnect::ArraySize 
    //              and TDDVideoPortConnect::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    TypeID
    // Description: The TypeID property defines GUID's indicating the sync 
    //              characteristics of the video port. 
    //              Possible values of this property are not limited to those 
    //              listed below, they are merely those that are predefined. 
    // Flags:       DDVPTYPE_BROOKTREE -
    //                The video sync information is embedded in the data 
    //                stream using the Brooktree definition. 
    //              DDVPTYPE_CCIR656 -
    //                The video sync information is embedded in the data 
    //                stream according to the CCIR656 specification. 
    //              DDVPTYPE_E_HREFH_VREFH -
    //                The video sync is provided externally with the HREF 
    //                indicated by an active high and the VREF indicated by 
    //                an active high. 
    //              DDVPTYPE_E_HREFH_VREFL -
    //                The video sync is provided externally with the HREF 
    //                indicated by an active high and the VREF is indicated 
    //                by an active low. 
    //              DDVPTYPE_E_HREFL_VREFH -
    //                The video sync is provided externally with the HREF 
    //                indicated by an active low and the VREF is indicated by 
    //                an active high. 
    //              DDVPTYPE_E_HREFL_VREFL -
    //                The video sync is provided externally with the HREF 
    //                indicated by an active low and the VREF is indicated by 
    //                an active low. 
    //              DDVPTYPE_PHILIPS -
    //                The video sync information is embedded in the data 
    //                stream using the Philips definition. 
    // ----------------------------------------------------------------------
    __property GUID TypeID[dword pArrayIndex] = { read=FGetTypeID, 
    write=FSetTypeID }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDVideoPortConnect's internal DDVIDEOPORTCONNECT 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TDDVideoPortConnect::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal DDVIDEOPORTCONNECT structures of the 
    //              TDDVideoPortConnect component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings[dword pArrayIndex] = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating the video 
    //              port connection's capabilities. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortConnect_Flags_Set Flags_[dword pArrayIndex] = { 
    read=FGetFlags_, write=FSetFlags_ }; 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDVideoPortConnect(TComponent* Owner); 
    virtual __fastcall ~TDDVideoPortConnect(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDVIDEOPORTCONNECT* Internal_DDVIDEOPORTCONNECT_Ptr = { 
    read=FGetInternal_DDVIDEOPORTCONNECT_Ptr, nodefault }; 

    void __fastcall Internal_DDVIDEOPORTCONNECT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDDVideoPortConnect_Flags_Set __fastcall FGetFlags_( dword pArrayIndex ); 
    void __fastcall FSetFlags_( dword pArrayIndex, 
    TDDVideoPortConnect_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags( dword pArrayIndex ); 
    void __fastcall FSetFlags( dword pArrayIndex, dword pFlags ); 
    dword __fastcall FGetPortWidth( dword pArrayIndex ); 
    void __fastcall FSetPortWidth( dword pArrayIndex, dword pPortWidth ); 
    dword __fastcall FGetSize( dword pArrayIndex ); 
    void __fastcall FSetSize( dword pArrayIndex, dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    GUID __fastcall FGetTypeID( dword pArrayIndex ); 
    void __fastcall FSetTypeID( dword pArrayIndex, GUID pTypeID ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings( dword pArrayIndex );

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDVIDEOPORTCONNECT* __fastcall FGetInternal_DDVIDEOPORTCONNECT_Ptr(); 
    void __fastcall FSetInternal_DDVIDEOPORTCONNECT( dword pArrayIndex, 
    DDVIDEOPORTCONNECT* pDDVIDEOPORTCONNECT ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSizeAll; 
    GUID fTypeID; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDVIDEOPORTCONNECT* fDDVIDEOPORTCONNECT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
