#ifndef TDDVideoPortDescH
#define TDDVideoPortDescH

// ==========================================================================
// File:         TDDVideoPortDesc.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDDVideoPortDesc Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDDVideoPortDesc properties.
#include "TDDVideoPortConnect.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDDVideoPortDesc
// Description: The TDDVideoPortDesc component wraps the DirectX 
//              DDVIDEOPORTDESC structure, which describes a video port. 
//              
//              It is used by TDx_DrawVideoPortContainer::CreateVideoPort. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDDVideoPortDesc : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDDVideoPortDesc : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    FieldHeight
    // Description: The FieldHeight property defines the field height, in 
    //              scan lines, of the odd and even fields of the incoming 
    //              video stream. 
    // ----------------------------------------------------------------------
    __property dword FieldHeight = { read=FGetFieldHeight, 
    write=FSetFieldHeight, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FieldWidth
    // Description: The FieldWidth property defines the width, in pixels, of 
    //              the incoming video stream. 
    // ----------------------------------------------------------------------
    __property dword FieldWidth = { read=FGetFieldWidth, 
    write=FSetFieldWidth, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MaxPixelsPerSecond
    // Description: The MaxPixelsPerSecond property defines the maximum 
    //              number of pixels per second the video port will allow. 
    // ----------------------------------------------------------------------
    __property dword MaxPixelsPerSecond = { read=FGetMaxPixelsPerSecond, 
    write=FSetMaxPixelsPerSecond, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    MicrosecondsPerField
    // Description: The MicrosecondsPerField property defines the time 
    //              interval, in microseconds, between live video VREF's. 
    //              This value should be rounded up to the nearest 
    //              microsecond. 
    // ----------------------------------------------------------------------
    __property dword MicrosecondsPerField = { read=FGetMicrosecondsPerField, 
    write=FSetMicrosecondsPerField, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DDVIDEOPORTDESC) }; 

    // ----------------------------------------------------------------------
    // Property:    VBIWidth
    // Description: The VBIWidth property defines the width, in pixels, of 
    //              the VBI data contained in the incoming video stream. 
    // ----------------------------------------------------------------------
    __property dword VBIWidth = { read=FGetVBIWidth, write=FSetVBIWidth, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VideoPortID
    // Description: The VideoPortID property defines the ID of the video port 
    //              that is to be used. 
    //              This is the zero based reference to the physical video 
    //              port. 
    // ----------------------------------------------------------------------
    __property dword VideoPortID = { read=FGetVideoPortID, 
    write=FSetVideoPortID, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VideoPortType
    // Description: The VideoPortType property references a 
    //              TDDVideoPortConnect component describing the connection 
    //              characteristics of the video port. 
    // ----------------------------------------------------------------------
    __property TDDVideoPortConnect* VideoPortType = { read=FGetVideoPortType, 
    write=FSetVideoPortType, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDDVideoPortDesc component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDDVideoPortDesc method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDDVideoPortDesc's internal DDVIDEOPORTDESC 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDDVideoPortDesc(TComponent* Owner); 
    virtual __fastcall ~TDDVideoPortDesc(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DDVIDEOPORTDESC* Internal_DDVIDEOPORTDESC_Ptr = { 
    read=FGetInternal_DDVIDEOPORTDESC_Ptr, nodefault }; 

    void __fastcall Internal_DDVIDEOPORTDESC_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFieldHeight(); 
    void __fastcall FSetFieldHeight( dword pFieldHeight ); 
    dword __fastcall FGetFieldWidth(); 
    void __fastcall FSetFieldWidth( dword pFieldWidth ); 
    dword __fastcall FGetMaxPixelsPerSecond(); 
    void __fastcall FSetMaxPixelsPerSecond( dword pMaxPixelsPerSecond ); 
    dword __fastcall FGetMicrosecondsPerField(); 
    void __fastcall FSetMicrosecondsPerField( dword pMicrosecondsPerField ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetVBIWidth(); 
    void __fastcall FSetVBIWidth( dword pVBIWidth ); 
    dword __fastcall FGetVideoPortID(); 
    void __fastcall FSetVideoPortID( dword pVideoPortID ); 
    TDDVideoPortConnect* __fastcall FGetVideoPortType(); 
    void __fastcall FSetVideoPortType( TDDVideoPortConnect* pVideoPortType ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DDVIDEOPORTDESC* __fastcall FGetInternal_DDVIDEOPORTDESC_Ptr(); 
    void __fastcall FSetInternal_DDVIDEOPORTDESC( DDVIDEOPORTDESC* 
    pDDVIDEOPORTDESC ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    TDDVideoPortConnect* fVideoPortType; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DDVIDEOPORTDESC fDDVIDEOPORTDESC; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
