#ifndef TDx_DrawColorControlH
#define TDx_DrawColorControlH

// ==========================================================================
// File:         TDx_DrawColorControl.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDx_DrawColorControl Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_DrawColorControl component.
class TDx_Draw;
// --------------------------------------------------------------------------
// external classes used by TDx_DrawColorControl methods.
class TDDColorControl;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_DrawColorControl
// Description: The TDx_DrawColorControl component wraps the DirectX 
//              IDirectDrawColorControl interface, which allows the setting 
//              and retrieval of color controls. 
//              
//              It is used to retrieve or modify the display control settings 
//              associated with a surface, thus allowing modification of such 
//              things as brightness, color, saturation, hue and sharpness. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_DrawColorControl : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_DrawColorControl : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_DrawColorControl::Create() method after it has 
    //              successfully created the internal 
    //              LPDIRECTDRAWCOLORCONTROL used within the 
    //              TDx_DrawColorControl component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTDRAWCOLORCONTROL 
    //              interface. 
    //              
    //              For example, you could retrieve the TDx_DrawColorControl 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_DrawColorControl::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_DrawColorControl::Destroy() method, just before the 
    //              TDx_DrawColorControl internal LPDIRECTDRAWCOLORCONTROL 
    //              interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_DrawColorControl::OnCreate() event. For example, 
    //              assume you have already setup the 
    //              TDx_DrawColorControl::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_DrawColorControl::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_DrawColorControl component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTDRAWCOLORCONTROL used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTDRAWCOLORCONTROL, call the 
    //              TDx_DrawColorControl::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_DrawColorControl method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTDRAWCOLORCONTROL interface used in the 
    //              TDx_DrawColorControl component and must be called before 
    //              any methods of the TDx_DrawColorControl component will 
    //              function. 
    // Params:      pDraw - 
    //                A TDX_Draw object responsible for the created 
    //                ColorControl. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_Draw* pDraw ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTDRAWCOLORCONTROL interface used in the 
    //              TDx_DrawColorControl component and should be called when 
    //              the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetColorControls()
    // Description: The TDx_DrawColorControl::GetColorControls method will 
    //              retrieve the current color control settings. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    // Params:      pColorControl - 
    //                The ColorControl parameter will reference the current 
    //                color control settings if this method returns 
    //                successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetColorControls( TDDColorControl* pColorControl 
    ); 

    // ----------------------------------------------------------------------
    // Method:      SetColorControls()
    // Description: The TDx_DrawColorControl::SetColorControls method will 
    //              set the color controls. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    // Params:      pColorControl - 
    //                The ColorControl parameter references the color control 
    //                settings that are to be set. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetColorControls( TDDColorControl* pColorControl 
    ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_DrawColorControl(TComponent* Owner); 
    virtual __fastcall ~TDx_DrawColorControl(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Draw_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTDRAWCOLORCONTROL Internal_LPDIRECTDRAWCOLORCONTROL = { 
    read=FGetInternal_LPDIRECTDRAWCOLORCONTROL, 
    write=FSetInternal_LPDIRECTDRAWCOLORCONTROL, nodefault }; 
    __property LPDIRECTDRAWCOLORCONTROL* 
    Internal_LPDIRECTDRAWCOLORCONTROL_Ptr = { 
    read=FGetInternal_LPDIRECTDRAWCOLORCONTROL_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTDRAWCOLORCONTROL_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTDRAWCOLORCONTROL __fastcall 
    FGetInternal_LPDIRECTDRAWCOLORCONTROL(); 
    void __fastcall FSetInternal_LPDIRECTDRAWCOLORCONTROL( 
    LPDIRECTDRAWCOLORCONTROL pLPDIRECTDRAWCOLORCONTROL ); 
    LPDIRECTDRAWCOLORCONTROL* __fastcall 
    FGetInternal_LPDIRECTDRAWCOLORCONTROL_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTDRAWCOLORCONTROL fLPDIRECTDRAWCOLORCONTROL; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
