#ifndef TDx_DrawGammaControlH
#define TDx_DrawGammaControlH

// ==========================================================================
// File:         TDx_DrawGammaControl.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDx_DrawGammaControl Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_DrawGammaControl component.
class TDx_Draw;
// --------------------------------------------------------------------------
// external classes used by TDx_DrawGammaControl methods.
class TDDGammaRamp;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_DrawGammaControl
// Description: The TDx_DrawGammaControl component wraps the 
//              IDirectDrawGammaControl interface, which can be used to set 
//              and retrieve the red, green and blue gamma ramp levels for 
//              the primary surface. 
//              
//              This component requires hardware support to be useful. 
//              Support can be checked by using TDx_Draw::GetCaps() and 
//              checking TDDCaps::Caps2 for the presence of 
//              DDCAPS2_PRIMARYGAMMA. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_DrawGammaControl : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_DrawGammaControl : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_DrawGammaControl::Create() method after it has 
    //              successfully created the internal 
    //              LPDIRECTDRAWGAMMACONTROL used within the 
    //              TDx_DrawGammaControl component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTDRAWGAMMACONTROL 
    //              interface. 
    //              
    //              For example, you could retrieve the TDx_DrawGammaControl 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_DrawGammaControl::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_DrawGammaControl::Destroy() method, just before the 
    //              TDx_DrawGammaControl internal LPDIRECTDRAWGAMMACONTROL 
    //              interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_DrawGammaControl::OnCreate() event. For example, 
    //              assume you have already setup the 
    //              TDx_DrawGammaControl::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_DrawGammaControl::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_DrawGammaControl component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTDRAWGAMMACONTROL used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTDRAWGAMMACONTROL, call the 
    //              TDx_DrawGammaControl::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_DrawGammaControl method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTDRAWGAMMACONTROL interface used in the 
    //              TDx_DrawGammaControl component and must be called before 
    //              any methods of the TDx_DrawGammaControl component will 
    //              function. 
    // Params:      pDraw - 
    //                The TDx_Draw object that will manage the created 
    //                GammaControl. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_Draw* pDraw ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTDRAWGAMMACONTROL interface used in the 
    //              TDx_DrawGammaControl component and should be called when 
    //              the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetGammaRamp()
    // Description: The TDx_DrawGammaControl::GetGammaRamp method will 
    //              retrieve the gamma information from a primary surface. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_EXCEPTION 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pRampData - 
    //                The RampData parameter references a TDDGammaRamp 
    //                component for holding the retrieved gamma information if 
    //                this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetGammaRamp( TDDGammaRamp* pRampData ); 

    // ----------------------------------------------------------------------
    // Method:      SetGammaRamp()
    // Description: The TDx_DrawGammaControl::SetGammaRamp method will set 
    //              the gamma information of the primary surface to the 
    //              specified values. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_EXCEPTION 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_OUTOFMEMORY 
    // Params:      pFlags - 
    //                The Flags parameter defines a flag indicating whether 
    //                gamma calibration should be used. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDSGR_CALIBRATE -
    //                    Adjust the gamma settings to suit the physical 
    //                    display so that different systems will display the 
    //                    same result.  Hardware support for gamma 
    //                    calibration must exist when using this flag. 
    //                    Support can be checked by using the 
    //                    TDx_Draw::GetCaps() method and checking the Caps2 
    //                    property for the DDCAPS2_PRIMARYGAMMA flag. 
    //              pRampData - 
    //                The RampData parameter references a TDDGammaRamp 
    //                component holding the new gamma values to be set for the 
    //                primary surface. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetGammaRamp( dword pFlags, TDDGammaRamp* 
    pRampData ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_DrawGammaControl(TComponent* Owner); 
    virtual __fastcall ~TDx_DrawGammaControl(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Draw_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTDRAWGAMMACONTROL Internal_LPDIRECTDRAWGAMMACONTROL = { 
    read=FGetInternal_LPDIRECTDRAWGAMMACONTROL, 
    write=FSetInternal_LPDIRECTDRAWGAMMACONTROL, nodefault }; 
    __property LPDIRECTDRAWGAMMACONTROL* 
    Internal_LPDIRECTDRAWGAMMACONTROL_Ptr = { 
    read=FGetInternal_LPDIRECTDRAWGAMMACONTROL_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTDRAWGAMMACONTROL_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTDRAWGAMMACONTROL __fastcall 
    FGetInternal_LPDIRECTDRAWGAMMACONTROL(); 
    void __fastcall FSetInternal_LPDIRECTDRAWGAMMACONTROL( 
    LPDIRECTDRAWGAMMACONTROL pLPDIRECTDRAWGAMMACONTROL ); 
    LPDIRECTDRAWGAMMACONTROL* __fastcall 
    FGetInternal_LPDIRECTDRAWGAMMACONTROL_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTDRAWGAMMACONTROL fLPDIRECTDRAWGAMMACONTROL; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
