#ifndef TDx_DrawPaletteH
#define TDx_DrawPaletteH

// ==========================================================================
// File:         TDx_DrawPalette.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDx_DrawPalette Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_DrawPalette component.
class TDx_Draw;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_DrawPalette
// Description: The TDx_DrawPalettePalette component wraps the DirectX 
//              IDirectDrawPalette interface, which is used for creating 
//              palette's and working with system level variables. 
//              
//              It is used to initialize manually created palette's, retrieve 
//              the palette's capabilities and set and retrieve the palette's 
//              color table entries. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_DrawPalette : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_DrawPalette : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_DrawPalette::Create() method after it has 
    //              successfully created the internal LPDIRECTDRAWPALETTE 
    //              used within the TDx_DrawPalette component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTDRAWPALETTE interface. 
    //              
    //              For example, you could retrieve the TDx_DrawPalette 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_DrawPalette::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_DrawPalette::Destroy() method, just before the 
    //              TDx_DrawPalette internal LPDIRECTDRAWPALETTE interface is 
    //              destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_DrawPalette::OnCreate() 
    //              event. For example, assume you have already setup the 
    //              TDx_DrawPalette::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_DrawPalette::OnDestroy() event to deallocate memory, 
    //              Destroy() the dependant components and, when required, 
    //              destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_DrawPalette component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTDRAWPALETTE used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTDRAWPALETTE, call the 
    //              TDx_DrawPalette::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_DrawPalette method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTDRAWPALETTE interface used in the 
    //              TDx_DrawPalette component and must be called before any 
    //              methods of the TDx_DrawPalette component will function. 
    // Params:      pFlags - 
    //                Flags describing the created palette. 
    //                Flags:
    //                  DDPCAPS_1BIT -
    //                    The DDPCAPS_1BIT Flag indicates the index is one 
    //                    bit. ie, there are two entrys in the palette. 
    //                  DDPCAPS_2BIT -
    //                    The DDPCAPS_2BIT Flag indicates the index is two 
    //                    bits. ie, there are four entrys in the palette. 
    //                  DDPCAPS_4BIT -
    //                    The DDPCAPS_4BIT Flag indicates the index is four 
    //                    bits. ie, there are sixteen entrys in the palette. 
    //                  DDPCAPS_8BIT -
    //                    The DDPCAPS_4BIT Flag indicates the index is eight 
    //                    bits. ie, there are 256 entrys in the palette. 
    //                  DDPCAPS_8BITENTRIES -
    //                    This flag indicates the index refers to an 8 bit 
    //                    color index. Valid only with the DDPCAPS_1BIT, 
    //                    DDPCAPS_2BIT, or DDPCAPS_4BIT flags, when the 
    //                    target surface is 8 bpp.   
    //                    
    //                    Each color entry is one byte long and is an index 
    //                    to a destination surfaces 8-bpp palette. 
    //                  DDPCAPS_ALLOW256 -
    //                    All 256 palette entries can be defined. 
    //                  DDPCAPS_ALPHA -
    //                    Used with 'texture' surfaces, this flag indicates 
    //                    the Flags member of the PALETTEENTRY structure is 
    //                    interpreted as a single 8 bit alpha value. 
    //                  DDPCAPS_INITIALIZE -
    //                    The created palette is to be initialized with the 
    //                    colors described in the ColorArray parameter. 
    //                  DDPCAPS_PRIMARYSURFACE -
    //                    The palette is for the primary surface, any changes 
    //                    will be immediatly visible on the display, unless 
    //                    DDPSETPAL_VSYNC is specified and supported. 
    //                  DDPCAPS_PRIMARYSURFACELEFT -
    //                    The palette is for the left eye primary surface, 
    //                    any changes will be immediatly visible on the left 
    //                    eye display, unless DDPSETPAL_VSYNC is specified 
    //                    and supported. 
    //                  DDPCAPS_VSYNC -
    //                    The palette can have modifications synchronised to 
    //                    the monitors refresh rate. 
    //              pColorArray - 
    //                An array of 2, 4, 16, or 256 PALETTEENTRY structures that 
    //                will initialize this DirectDrawPalette object 
    //              pDraw - 
    //                A TDx_Draw object that is responsible for the created 
    //                palette 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( dword pFlags, PALETTEENTRY* pColorArray, 
    TDx_Draw* pDraw ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTDRAWPALETTE interface used in the 
    //              TDx_DrawPalette component and should be called when the 
    //              internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      GetCaps()
    // Description: The TDx_DrawPalette::GetCaps method will return the 
    //              palette capabilities. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pCaps - 
    //                The Caps parameter will reference the palette 
    //                capabilities if this method returns successfully. 
    //                The described effect applies when the flag is set. 
    //                These flags are the same as those listed for 
    //                TDDCaps::PalCaps. 
    //                Flags:
    //                  DDPCAPS_1BIT -
    //                    The palette index is 1 bit. 
    //                    There will be 2 color table entries. 
    //                  DDPCAPS_2BIT -
    //                    The palette index is 2 bit. 
    //                    There will be 4 color table entries. 
    //                  DDPCAPS_4BIT -
    //                    The palette index is 4 bit. 
    //                    There will be 16 color table entries. 
    //                  DDPCAPS_8BIT -
    //                    The palette index is 8 bit. 
    //                    There will be 256 color table entries. 
    //                  DDPCAPS_8BITENTRIES -
    //                    The palette index is an index to a 8-bit color 
    //                    index. 
    //                    This flag is only valid when the target surface is 
    //                    8 bpp and either  DDPCAPS_1BIT, DDPCAPS_2BIT or 
    //                    DDPCAPS_4BIT is set. Color entries are 8 bits and 
    //                    are indexes into the 8 bpp palette of the 
    //                    destination surface. 
    //                  DDPCAPS_ALLOW256 -
    //                    The palette can have all 256 entries specified by 
    //                    the application. 
    //                    Entry 0 and entry 255 on 8-bit palettes are 
    //                    reserved for system use unless this flag is set. 
    //                  DDPCAPS_ALPHA -
    //                    The peFlags member of the associated PALETTEENTRY 
    //                    structure is to be interpreted as an 8-bit alpha 
    //                    value. 
    //                    Palettes utilizing this capability may only be 
    //                    attached to textures. 
    //                  DDPCAPS_PRIMARYSURFACE -
    //                    The palette is attached to the primary surface. 
    //                    Changes to the palette will be immediately apparent 
    //                    on the display unless DDPCAPS_VSYNC is set and 
    //                    supported. 
    //                  DDPCAPS_PRIMARYSURFACELEFT -
    //                    The palette is attached to the primary surface for 
    //                    the left eye. 
    //                    Changes to the palette will have an immediate 
    //                    effect on the left eye display unless DDPCAPS_VSYNC 
    //                    is set and supported. 
    //                  DDPCAPS_VSYNC -
    //                    The palette is capable of being modified 
    //                    synchronously with the monitor's refresh rate. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetCaps( dword* pCaps ); 

    // ----------------------------------------------------------------------
    // Method:      GetEntries()
    // Description: The TDx_DrawPalette::GetEntries method will retrieve 
    //              specified color table entries from the palette. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_NOTPALETTIZED 
    // Params:      pBase - 
    //                The Base parameter defines the start of the entries to be 
    //                retrieved sequentially. 
    //              pNumEntries - 
    //                The NumEntries parameter defines the number of entries 
    //                the Entries parameter of this method can hold. 
    //              pEntries - 
    //                The Entries parameter will reference the array of 
    //                retrieved PALETTEENTRY 's if this method returns 
    //                successfully. 
    //                Palette entries will be 4 bytes each unless 
    //                DDPCAPS_8BITENTRIES is set, in which case each entry will 
    //                be 1 byte. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetEntries( dword pBase, dword pNumEntries, 
    PALETTEENTRY* pEntries ); 

    // ----------------------------------------------------------------------
    // Method:      LoadFromFile()
    // Description: The LoadFromFile method is used to load a 256-color 
    //              palette from an existing .BMP image file. 
    // Params:      pDraw - 
    //                The Draw parameter must be a previously created TDx_Draw 
    //                component. 
    //              pFileName - 
    //                The FileName parameter specifies the name of a standard 
    //                .BMP file that the palette is to be loaded from and can 
    //                include a directory path if required. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall LoadFromFile( TDx_Draw* pDraw, AnsiString 
    pFileName ); 

    // ----------------------------------------------------------------------
    // Method:      LoadFromResource()
    // Description: The LoadFromResource method is used to load a 256-color 
    //              palette from an existing bitmap resource. 
    // Params:      pDraw - 
    //                The Draw parameter must be a previously created TDx_Draw 
    //                component. 
    //              pResourceName - 
    //                The ResourceName parameter specifies the name of a 
    //                standard bitmap source that the palette is to be loaded 
    //                from. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall LoadFromResource( TDx_Draw* pDraw, AnsiString 
    pResourceName ); 

    // ----------------------------------------------------------------------
    // Method:      SetEntries()
    // Description: The TDx_DrawPalette::SetEntries method will change the 
    //              color entries in a palette. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_NOPALETTEATTACHED 
    //              DDERR_NOTPALETTIZED 
    //              DDERR_UNSUPPORTED 
    // Params:      pStartingEntry - 
    //                The StartingEntry parameter defines the palette index of 
    //                the first entry to be set. 
    //              pCount - 
    //                The Count parameter defines the number of palette entries 
    //                to be set. 
    //              pEntries - 
    //                The Entries parameter references the PALETTEENTRY 's that 
    //                are to replace those being changed. 
    //                Palette entries will be 4 bytes each unless 
    //                DDPCAPS_8BITENTRIES is set, in which case each entry will 
    //                be 1 byte. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetEntries( dword pStartingEntry, dword pCount, 
    PALETTEENTRY* pEntries ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_DrawPalette(TComponent* Owner); 
    virtual __fastcall ~TDx_DrawPalette(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Draw_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTDRAWPALETTE Internal_LPDIRECTDRAWPALETTE = { 
    read=FGetInternal_LPDIRECTDRAWPALETTE, 
    write=FSetInternal_LPDIRECTDRAWPALETTE, nodefault }; 
    __property LPDIRECTDRAWPALETTE* Internal_LPDIRECTDRAWPALETTE_Ptr = { 
    read=FGetInternal_LPDIRECTDRAWPALETTE_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTDRAWPALETTE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTDRAWPALETTE __fastcall FGetInternal_LPDIRECTDRAWPALETTE(); 
    void __fastcall FSetInternal_LPDIRECTDRAWPALETTE( LPDIRECTDRAWPALETTE 
    pLPDIRECTDRAWPALETTE ); 
    LPDIRECTDRAWPALETTE* __fastcall FGetInternal_LPDIRECTDRAWPALETTE_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTDRAWPALETTE fLPDIRECTDRAWPALETTE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
