#ifndef TDx_DrawVideoPortH
#define TDx_DrawVideoPortH

// ==========================================================================
// File:         TDx_DrawVideoPort.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDx_DrawVideoPort Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_DrawVideoPort component.
class TDx_DrawVideoPortContainer;
class TDDVideoPortDesc;
// --------------------------------------------------------------------------
// external classes used by TDx_DrawVideoPort methods.
class TDx_DrawSurface;
class TDDPixelFormat;
class TDDVideoPortBandwidth;
class TDDColorControl;
class TDDVideoPortInfo;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDx_DrawVideoPort
// Description: The TDx_DrawVideoPort component wraps the DirectX 
//              IDirectDrawVideoPort interface, which channels live video 
//              from the video port hardware to a surface, often an overlay, 
//              for display. 
//              
//              It is used to start and stop the flow of video information, 
//              retrieve the input and output formats supported, retrieve the 
//              zoom factors required to display video successfully, change 
//              the video transfer parameters and retrieve the number of the 
//              video line currently being written to the frame buffer. 
//              
//              Other methods include the retrieval of status, polarity or 
//              color control information, assigning target surfaces for the 
//              video, changing the flipping behavior, setting color controls 
//              and waiting for the vsync or a specified scan line. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_DrawVideoPort : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_DrawVideoPort : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_DrawVideoPort::Create() method after it has 
    //              successfully created the internal LPDIRECTDRAWVIDEOPORT 
    //              used within the TDx_DrawVideoPort component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTDRAWVIDEOPORT 
    //              interface. 
    //              
    //              For example, you could retrieve the TDx_DrawVideoPort 
    //              component's capabilities, Create() dependant components, 
    //              allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_DrawVideoPort::OnDestroy() event, you can fully 
    //              automate and insulate your code on a per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_DrawVideoPort::Destroy() method, just before the 
    //              TDx_DrawVideoPort internal LPDIRECTDRAWVIDEOPORT 
    //              interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_DrawVideoPort::OnCreate() 
    //              event. For example, assume you have already setup the 
    //              TDx_DrawVideoPort::OnCreate() event to retrieve the 
    //              component's capabilities, Create() a dependant component 
    //              and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_DrawVideoPort::OnDestroy() event to deallocate 
    //              memory, Destroy() the dependant components and, when 
    //              required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_DrawVideoPort component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTDRAWVIDEOPORT used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTDRAWVIDEOPORT, call the 
    //              TDx_DrawVideoPort::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_DrawVideoPort method or 
    //              fget/fset. eg. DD_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTDRAWVIDEOPORT interface used in the 
    //              TDx_DrawVideoPort component and must be called before any 
    //              methods of the TDx_DrawVideoPort component will function. 
    // Params:      pFlags - 
    //                Flags specifying video-port control options. This 
    //                parameter can be NULL if control options are not needed: 
    //                Flags:
    //                  DDVPCREATE_VBIONLY -
    //                    The process wants to obtain control of the VBI 
    //                    period of the video stream. 
    //                  DDVPCREATE_VIDEOONLY -
    //                    The process wants to obtain control of the video 
    //                    period of the video stream. 
    //              pVideoPortDesc - 
    //                The VideoPortDesc parameter describes the videoport to be 
    //                created. 
    //              pVideoPortContainer - 
    //                The VideoPortContainer that will manage the created 
    //                videoport. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( dword pFlags, TDDVideoPortDesc* 
    pVideoPortDesc, TDx_DrawVideoPortContainer* pVideoPortContainer ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTDRAWVIDEOPORT interface used in the 
    //              TDx_DrawVideoPort component and should be called when the 
    //              internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      Flip()
    // Description: The TDx_DrawVideoPort::Flip method will cause the video 
    //              port to use a new surface for the next frame of video 
    //              data. 
    //              
    //              This can be useful for preventing artifacts such as 
    //              tearing. 
    //              
    //              Calls to TDx_DrawVideoPort::Flip() are asynchronous but 
    //              the flips generated are synchronized with the video 
    //              signals VBI. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pSurface - 
    //                The Surface parameter references a surface to receive the 
    //                next video frame. 
    //                Set this parameter to NULL to cycle through the flip 
    //                chain in the normal attachment order. 
    //              pFlags - 
    //                The Flags parameter defines flags indicating the flip 
    //                options. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDVPFLIP_VBI -
    //                    The surface referenced by the Surface property of 
    //                    this method will receive VBI video data. 
    //                  DDVPFLIP_VIDEO -
    //                    The surface referenced by the Surface property of 
    //                    this method will receive normal video data. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Flip( TDx_DrawSurface* pSurface, dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      GetBandwidthInfo()
    // Description: The TDx_DrawVideoPort::GetBandwidthInfo method will 
    //              retrieve the overlay zoom factors and device limitations 
    //              for a video port outputting the specified pixel format. 
    //              
    //              This method is usually called twice. Once with DDVPB_TYPE 
    //              set to determine whether the bandwidth information is 
    //              provided in terms of the destination or source overlay 
    //              size and again with either of the other flags set as 
    //              indicated by the previous call to retrieve the required 
    //              information. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pFormat - 
    //                The Format parameter references a TDDPixelFormat 
    //                component defining the output pixel format for which 
    //                bandwidth information is being requested. 
    //              pWidth - 
    //                The Width parameter defines the horizontal dimension of 
    //                an overlay or the video data width. 
    //                Interpretation of this value depends on which flags are 
    //                set in the Flags parameter of this method. 
    //              pHeight - 
    //                The Height parameter defines the vertical dimension of an 
    //                overlay or the video data height. 
    //                Interpretation of this value depends on which flags are 
    //                set in the Flags parameter of this method. 
    //              pFlags - 
    //                The Flags parameter defines flags indicating how the 
    //                Width and Height parameters of this method are to be 
    //                interpreted. The described effect applies when the flag 
    //                is set. 
    //                Flags:
    //                  DDVPB_OVERLAY -
    //                    The Width and Height parameters of this method 
    //                    define the source overlay surface's size. 
    //                    This flag should be set when the video port is 
    //                    dependent on the source overlay size. 
    //                  DDVPB_TYPE -
    //                    Retrieve the device's dependency type in the 
    //                    TDDVideoPortBandwidth::Caps referenced by the 
    //                    Bandwidth parameter of this method. 
    //                    This flag should be used on the first call to this 
    //                    method. 
    //                    The Width and Height parameters of this method will 
    //                    be empty. 
    //                  DDVPB_VIDEOPORT -
    //                    The Width and Height parameters define the prescale 
    //                    size of the video data written to the frame buffer 
    //                    by the video port. 
    //                    This flag should be set when the video port is 
    //                    dependent on the overlay zoom factor. 
    //              pBandwidth - 
    //                The Bandwidth parameter references a 
    //                TDDVideoPortBandwidth component for holding the retrieved 
    //                bandwidth and device dependency information if this 
    //                method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetBandwidthInfo( TDDPixelFormat* pFormat, dword 
    pWidth, dword pHeight, dword pFlags, TDDVideoPortBandwidth* pBandwidth ); 

    // ----------------------------------------------------------------------
    // Method:      GetColorControls()
    // Description: The TDx_DrawVideoPort::GetColorControls method will 
    //              retrieve the video port's current color control settings. 
    //              
    //              The color control options supported by the video port can 
    //              be determined from TDDColorControl::Flags. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    // Params:      pColorControl - 
    //                The ColorControl parameter references a TDDColorControl 
    //                component for holding the video port's current color 
    //                control settings if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetColorControls( TDDColorControl* pColorControl 
    ); 

    // ----------------------------------------------------------------------
    // Method:      GetFieldPolarity()
    // Description: The TDx_DrawVideoPort::GetFieldPolarity method will 
    //              indicate if the current field of interlaced video is odd 
    //              or even. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    //              DDERR_VIDEONOTACTIVE 
    // Params:      pFieldPolarity - 
    //                The FieldPolarity parameter references a boolean that 
    //                will be set to TRUE if the current field is even and 
    //                FALSE if the current field is odd. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetFieldPolarity( bool* pFieldPolarity ); 

    // ----------------------------------------------------------------------
    // Method:      GetInputFormats()
    // Description: The TDx_DrawVideoPort::GetInputFormats method will 
    //              retrieve the supported video port input pixel formats. 
    //              
    //              Setting the Formats parameter to NULL will cause the 
    //              number of supported input formats to be returned in the 
    //              NumFormats parameter. 
    //              
    //              If this method call fails, the one of the following 
    //              values will be returned: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_MOREDATA 
    // Params:      pNumFormats - 
    //                The NumFormats parameter references the size of the array 
    //                referenced by the Formats parameter of this method. 
    //                When the size needed exceeds this value, the array will 
    //                be filled, this parameter set to the actual size needed 
    //                and a DDERR_MOREDATA error returned. 
    //              pFormats - 
    //                The Formats parameter references an array of 
    //                TDDPixelFormat components for holding the retrieved input 
    //                pixel formats if this method returns successfully. 
    //                Setting this parameter to NULL will cause the required 
    //                array size to be returned in the NumFormats parameter of 
    //                this method. 
    //              pFlags - 
    //                The Flags parameter defines flags indicating which 
    //                portion of the video signal to enumerate formats for. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDVPFORMAT_VBI -
    //                    Retrieve the format of the VBI data. 
    //                  DDVPFORMAT_VIDEO -
    //                    Retrieve the format of the normal video data. 
    // ----------------------------------------------------------------------
    virtual HRESULT __fastcall GetInputFormats( dword* pNumFormats, 
    TDDPixelFormat* pFormats, dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      GetOutputFormats()
    // Description: The TDx_DrawVideoPort::GetOutputFormats method will 
    //              retrieve the supported video port output pixel formats 
    //              for a specified input format. 
    //              
    //              If this method call fails, the one of the following 
    //              values will be returned: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_MOREDATA 
    // Params:      pInputFormat - 
    //                The InputFormat parameter references a TDDPixelFormat 
    //                component holding the input format for which conversion 
    //                information is desired. 
    //              pNumFormats - 
    //                The NumFormats parameter references the size of the array 
    //                referenced by the Formats parameter of this method. 
    //                When the size needed exceeds this value, the array will 
    //                be filled, this parameter set to the actual size needed 
    //                and a DDERR_MOREDATA error returned. 
    //              pFormats - 
    //                The Formats parameter references an array of 
    //                TDDPixelFormat components for holding the retrieved 
    //                output pixel formats if this method returns successfully. 
    //                Setting this parameter to NULL will cause the required 
    //                array size to be returned in the NumFormats parameter of 
    //                this method. 
    //              pFlags - 
    //                The Flags parameter defines flags indicating the portion 
    //                of the video signal for which the format is to be 
    //                enumerated. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDVPFORMAT_VBI -
    //                    Retrieve the format of the VBI data. 
    //                  DDVPFORMAT_VIDEO -
    //                    Retrieve the format of the normal video data. 
    // ----------------------------------------------------------------------
    virtual HRESULT __fastcall GetOutputFormats( TDDPixelFormat* 
    pInputFormat, dword* pNumFormats, TDDPixelFormat* pFormats, dword pFlags 
    ); 

    // ----------------------------------------------------------------------
    // Method:      GetVideoLine()
    // Description: The TDx_DrawVideoPort::GetVideoLine method will retrieve 
    //              the scan line number currently being written to the frame 
    //              buffer. 
    //              
    //              This number is the true video line, relative to the field 
    //              height, before prescaling. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    //              DDERR_VERTICALBLANKINPROGRESS 
    //              DDERR_VIDEONOTACTIVE 
    // Params:      pLine - 
    //                The Line parameter will reference the current scan line 
    //                number if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetVideoLine( dword* pLine ); 

    // ----------------------------------------------------------------------
    // Method:      GetVideoSignalStatus()
    // Description: The TDx_DrawVideoPort::GetVideoSignalStatus method will 
    //              determine the current video signal status. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    // Params:      pStatus - 
    //                The Status parameter references a variable defining flags 
    //                indicating the current video signal status. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDVPSQ_NOSIGNAL -
    //                    There is no video signal coming to the video port. 
    //                  DDVPSQ_SIGNALOK -
    //                    There is a valid video signal coming to the video 
    //                    port. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetVideoSignalStatus( dword* pStatus ); 

    // ----------------------------------------------------------------------
    // Method:      SetColorControls()
    // Description: The TDx_DrawVideoPort::SetColorControls method will set 
    //              the video port color controls. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    // Params:      pColorControl - 
    //                The ColorControl parameter references a TDDColorControl 
    //                component holding the new video port color control 
    //                settings. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetColorControls( TDDColorControl* pColorControl 
    ); 

    // ----------------------------------------------------------------------
    // Method:      SetTargetSurface()
    // Description: The TDx_DrawVideoPort::SetTargetSurface method will set 
    //              the target surface for receiving video and/or VBI data. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pSurface - 
    //                The Surface parameter references the surface which will 
    //                receive video and/or VBI data. 
    //              pFlags - 
    //                The Flags parameter defines flags indicating the type of 
    //                target surface. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDVPTARGET_VBI -
    //                    The surface will receive VBI data only. 
    //                  DDVPTARGET_VIDEO -
    //                    The surface will receive both video and VBI data 
    //                    unless a separate surface for VBI data has been 
    //                    attached. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall SetTargetSurface( TDx_DrawSurface* pSurface, 
    dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      StartVideo()
    // Description: The TDx_DrawVideoPort::StartVideo method will start the 
    //              flow of live video data from the video port hardware into 
    //              the target surface. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_SURFACELOST 
    // Params:      pVideoInfo - 
    //                The VideoInfo parameter references a TDDVideoPortInfo 
    //                component defining the video transfer properties. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall StartVideo( TDDVideoPortInfo* pVideoInfo ); 

    // ----------------------------------------------------------------------
    // Method:      StopVideo()
    // Description: The TDx_DrawVideoPort::StopVideo method will stop the 
    //              sending of video data to the frame buffer. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with the following value: 
    //              DDERR_INVALIDOBJECT 
    // ----------------------------------------------------------------------
    virtual bool __fastcall StopVideo(); 

    // ----------------------------------------------------------------------
    // Method:      UpdateVideo()
    // Description: The TDx_DrawVideoPort::UpdateVideo method will modify the 
    //              transfer properties governing the flow of video data from 
    //              the hardware video port to a surface. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pVideoInfo - 
    //                The VideoInfo parameter references a TDDVideoPortInfo 
    //                component defining the video transfer properties. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall UpdateVideo( TDDVideoPortInfo* pVideoInfo ); 

    // ----------------------------------------------------------------------
    // Method:      WaitForSync()
    // Description: The TDx_DrawVideoPort::WaitForSync method can help an 
    //              application synchronise by waiting for the VSYNC or a 
    //              specified scan line. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    //              DDERR_VIDEONOTACTIVE 
    //              DDERR_WASSTILLDRAWING 
    // Params:      pFlags - 
    //                The Flags parameter defines flags indicating just how to 
    //                wait. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DDVPWAIT_BEGINVBLANK -
    //                    Stop waiting at the start of the VBI. 
    //                  DDVPWAIT_ENDVBLANK -
    //                    Stop waiting at the end of the VBI. 
    //                  DDVPWAIT_LINE -
    //                    Stop waiting when the line number specified in the 
    //                    Line parameter of this method is passed. 
    //              pLine - 
    //                The Line parameter defines the video line number, 
    //                relative to field height and before prescaling, that the 
    //                waiting should stop. 
    //                This parameter is only used when DDVPWAIT_LINE is set. 
    //              pTimeout - 
    //                The Timeout parameter defines the maximum time, in 
    //                milliseconds, to wait before timing out and returning. 
    //                Setting this property to 0 results in a wait of 3 times 
    //                the setting of TDDVideoPortDesc::MicrosecondsPerField. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall WaitForSync( dword pFlags, dword pLine, dword 
    pTimeout ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_DrawVideoPort(TComponent* Owner); 
    virtual __fastcall ~TDx_DrawVideoPort(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Draw_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTDRAWVIDEOPORT Internal_LPDIRECTDRAWVIDEOPORT = { 
    read=FGetInternal_LPDIRECTDRAWVIDEOPORT, 
    write=FSetInternal_LPDIRECTDRAWVIDEOPORT, nodefault }; 
    __property LPDIRECTDRAWVIDEOPORT* Internal_LPDIRECTDRAWVIDEOPORT_Ptr = { 
    read=FGetInternal_LPDIRECTDRAWVIDEOPORT_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTDRAWVIDEOPORT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTDRAWVIDEOPORT __fastcall FGetInternal_LPDIRECTDRAWVIDEOPORT(); 
    void __fastcall FSetInternal_LPDIRECTDRAWVIDEOPORT( LPDIRECTDRAWVIDEOPORT 
    pLPDIRECTDRAWVIDEOPORT ); 
    LPDIRECTDRAWVIDEOPORT* __fastcall 
    FGetInternal_LPDIRECTDRAWVIDEOPORT_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTDRAWVIDEOPORT fLPDIRECTDRAWVIDEOPORT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
