#ifndef TDx_DrawVideoPortContainerH
#define TDx_DrawVideoPortContainerH

// ==========================================================================
// File:         TDx_DrawVideoPortContainer.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons)
// Description:  This file defines the TDx_DrawVideoPortContainer Component 
//
// "TDx_Draw_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Draw_Library_Defns.H"
#include "TDx_Draw_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_DrawVideoPortContainer component.
class TDx_Draw;
// --------------------------------------------------------------------------
// external classes used by TDx_DrawVideoPortContainer methods.
class TDDVideoPortDesc;
class TDx_DrawVideoPort;
class TDDVideoPortCaps;
class TDDVideoPortConnect;
class TDDVideoPortStatus;

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// Definition for the OnEnumVideoPorts callback event.
typedef	__fastcall void	(__closure*     TDx_EnumVideoPortsCallback) 
                        ( TObject*      Sender, 
                          TDDVideoPortCaps* VideoPortCaps, 
                          void*         Context, 
                          bool&         Finished ); 

// ==========================================================================
// Class:       TDx_DrawVideoPortContainer
// Description: The TDx_DrawVideoPortContainer component wraps the DirectX 
//              IDDVideoPortContainer interface, which is used to create and 
//              manipulate TDx_DrawVideoPort components. 
//              
//              It is used to create and enumerate video ports, retrieve the 
//              status of the video port and get the video port's connection 
//              information. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_DrawVideoPortContainer : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_DrawVideoPortContainer : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_DrawVideoPortContainer::Create() method after it has 
    //              successfully created the internal LPDDVIDEOPORTCONTAINER 
    //              used within the TDx_DrawVideoPortContainer component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDDVIDEOPORTCONTAINER 
    //              interface. 
    //              
    //              For example, you could retrieve the 
    //              TDx_DrawVideoPortContainer component's capabilities, 
    //              Create() dependant components, allocate memory, etc. 
    //              
    //              When used in conjunction with the 
    //              TDx_DrawVideoPortContainer::OnDestroy() event, you can 
    //              fully automate and insulate your code on a per-component 
    //              basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_DrawVideoPortContainer::Destroy() method, just before 
    //              the TDx_DrawVideoPortContainer internal 
    //              LPDDVIDEOPORTCONTAINER interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the 
    //              TDx_DrawVideoPortContainer::OnCreate() event. For 
    //              example, assume you have already setup the 
    //              TDx_DrawVideoPortContainer::OnCreate() event to retrieve 
    //              the component's capabilities, Create() a dependant 
    //              component and allocate some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_DrawVideoPortContainer::OnDestroy() event to 
    //              deallocate memory, Destroy() the dependant components 
    //              and, when required, destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_DrawVideoPortContainer component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Callback:    OnEnumVideoPorts()
    // Description: The OnEnumVideoPorts callback is used to process 
    //              information returned by the 
    //              TDx_DrawVideoPortContainer::EnumVideoPorts method. 
    //              Set the Finished parameter to halt the enumeration 
    //              process. 
    // ----------------------------------------------------------------------
    __property TDx_EnumVideoPortsCallback OnEnumVideoPorts = 
    {read=FOnEnumVideoPorts, write=FOnEnumVideoPorts, nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDDVIDEOPORTCONTAINER used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDDVIDEOPORTCONTAINER, call the 
    //              TDx_DrawVideoPortContainer::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_DrawVideoPortContainer 
    //              method or fget/fset. eg. DD_OK or DDERR_SURFACELOST or 
    //              TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DD_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDDVIDEOPORTCONTAINER interface used in the 
    //              TDx_DrawVideoPortContainer component and must be called 
    //              before any methods of the TDx_DrawVideoPortContainer 
    //              component will function. 
    // Params:      pDraw - 
    //                The TDx_Draw object that will manage the created 
    //                VideoPortContainer. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create( TDx_Draw* pDraw ); 

    // ----------------------------------------------------------------------
    // Method:      CreateVideoPort()
    // Description: The TDx_DrawVideoPortContainer::CreateVideoPort method 
    //              will create a video port as defined by the settings of 
    //              the Flags and VideoPortDesc parameters. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_CURRENTLYNOTAVAIL 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_NOCOOPERATIVELEVELSET 
    //              DDERR_OUTOFCAPS 
    //              DDERR_OUTOFMEMORY 
    // Params:      pFlags - 
    //                The Flags parameter defines flags indicating which 
    //                portion of the video stream the video-port will control. 
    //                The described effect applies when the flag is set. 
    //                Set this parameteer to NULL if no control options are 
    //                needed. 
    //                Flags:
    //                  DDVPCREATE_VBIONLY -
    //                    The new video port will only control the VBI 
    //                    portion of the video stream. 
    //                  DDVPCREATE_VIDEOONLY -
    //                    The new video port will only control the non-VBI 
    //                    portion of the video stream. 
    //              pVideoPortDesc - 
    //                The VideoPortDesc parameter references a TDDVideoPortDesc 
    //                component defining the properties of the video port being 
    //                created. 
    //              pVideoPort - 
    //                The VideoPort parameter will reference the new video port 
    //                if this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CreateVideoPort( dword pFlags, TDDVideoPortDesc* 
    pVideoPortDesc, TDx_DrawVideoPort* pVideoPort ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDDVIDEOPORTCONTAINER interface used in the 
    //              TDx_DrawVideoPortContainer component and should be called 
    //              when the internal interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      EnumVideoPorts()
    // Description: The TDx_DrawVideoPortContainer::EnumVideoPorts method 
    //              will enumerate video ports that are compatible with the 
    //              specified capabilities. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    // Params:      pVideoPortCaps - 
    //                The VideoPortCaps parameter references a TDDVideoPortCaps 
    //                component defining the capabilities that are to be 
    //                checked against the available video ports. 
    //                When this parameter is set to NULL, all video ports will 
    //                be enumerated. 
    //              pContext - 
    //                The Context parameter references a caller defined object 
    //                that is to be passed to the 
    //                TDx_DrawVideoPortContainer::EnumVideoCallback() event for 
    //                each video port enumerated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall EnumVideoPorts( TDDVideoPortCaps* pVideoPortCaps, 
    void* pContext ); 

    // ----------------------------------------------------------------------
    // Method:      GetVideoPortConnectInfo()
    // Description: The TDx_DrawVideoPortContainer::GetVideoPortConnectInfo 
    //              method will retrieve connection information from the 
    //              specified video port. Before calling this method, make 
    //              sure to setup the TDDVideoPortConnect multiwrapper 
    //              component with appropriate data. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DSERR_INVALIDPARAM 
    //              DSERR_OUTOFMEMORY 
    // Params:      pPortId - 
    //                The PortId parameter defines the ID of the video port for 
    //                which connection information is to be retrieved. 
    //              pNumEntries - 
    //                The NumEntries parameter references the size of the array 
    //                referenced by the ConnectInfo parameter of this method. 
    //                When this value is less than the total connections, the 
    //                array is filled, this parameter is set to the total 
    //                number of connections and DDERR_MOREDATA returned in both 
    //                fErrorValue and the OnError() event. 
    //              pConnectInfo - 
    //                The ConnectInfo parameter references a 
    //                TDDVideoPortConnect multiwrapper component used for 
    //                holding the retrieved connection information if this 
    //                method returns successfully. 
    //                When this parameter is NULL, the NumEntries parameter of 
    //                this component will be set to the total number of 
    //                connections supported and true returned. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall GetVideoPortConnectInfo( dword pPortId, dword* 
    pNumEntries, TDDVideoPortConnect* pConnectInfo ); 

    // ----------------------------------------------------------------------
    // Method:      Internal_EnumVideoPortsCallback()
    // Description: The Internal_EnumVideoPortsCallback method is used 
    //              internally by the TDx_Draw_Library to redirect a standard 
    //              callback function to a BCB style event. 
    // Params:      pDDVideoPortCaps - 
    //                The DDVideoPortCaps parameter describes the currently 
    //                enumerated VideoPort. 
    //              pContext - 
    //                Application defined data passed to the enumeration method 
    //                call. 
    // ----------------------------------------------------------------------
    static HRESULT __stdcall Internal_EnumVideoPortsCallback( 
    LPDDVIDEOPORTCAPS pDDVideoPortCaps, LPVOID pContext ); 

    // ----------------------------------------------------------------------
    // Method:      QueryVideoPortStatus()
    // Description: The TDx_DrawVideoPortContainer::QueryVideoPortStatus 
    //              method will retrieve the status of the specified video 
    //              port. 
    //              
    //              If the method call fails, the OnError event will be 
    //              triggered with one of the following values: 
    //              DDERR_CURRENTLYNOTAVAIL 
    //              DDERR_EXCEPTION 
    //              DDERR_INVALIDOBJECT 
    //              DDERR_INVALIDPARAMS 
    //              DDERR_UNSUPPORTED 
    // Params:      pPortId - 
    //                The PortId parameter defines the ID of the video port for 
    //                which status information is to be retrieved. 
    //              pVPStatus - 
    //                The VPStatus parameter references a TDDVideoPortStatus 
    //                component for holding the retrieved status information if 
    //                this method returns successfully. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall QueryVideoPortStatus( dword pPortId, 
    TDDVideoPortStatus* pVPStatus ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_DrawVideoPortContainer(TComponent* Owner); 
    virtual __fastcall ~TDx_DrawVideoPortContainer(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Draw_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDDVIDEOPORTCONTAINER Internal_LPDDVIDEOPORTCONTAINER = { 
    read=FGetInternal_LPDDVIDEOPORTCONTAINER, 
    write=FSetInternal_LPDDVIDEOPORTCONTAINER, nodefault }; 
    __property LPDDVIDEOPORTCONTAINER* Internal_LPDDVIDEOPORTCONTAINER_Ptr = 
    { read=FGetInternal_LPDDVIDEOPORTCONTAINER_Ptr, nodefault }; 
    void __fastcall Internal_LPDDVIDEOPORTCONTAINER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDDVIDEOPORTCONTAINER __fastcall FGetInternal_LPDDVIDEOPORTCONTAINER(); 
    void __fastcall FSetInternal_LPDDVIDEOPORTCONTAINER( 
    LPDDVIDEOPORTCONTAINER pLPDDVIDEOPORTCONTAINER ); 
    LPDDVIDEOPORTCONTAINER* __fastcall 
    FGetInternal_LPDDVIDEOPORTCONTAINER_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDDVIDEOPORTCONTAINER fLPDDVIDEOPORTCONTAINER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

    // ----------------------------------------------------------------------
    // Callback Variables
    // ----------------------------------------------------------------------
    TDx_EnumVideoPortsCallback FOnEnumVideoPorts;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
